@@ -1,13 +1,12 @@
 /****************************************************************************
 ** ViewerView meta object code from reading C++ file 'viewer-qt.h'
 **
-** Created: Mon Apr 16 18:15:24 2001
-**      by: The Qt MOC ($Id: qt/src/moc/moc.y   2.2.3   edited 2000-12-11 $)
+** Created: Wed May 2 10:50:42 2001
+**      by: The Qt MOC ($Id: qt/src/moc/moc.y   2.3.0   edited 2001-02-20 $)
 **
 ** WARNING! All changes made in this file will be lost!
 *****************************************************************************/
 
-#define Q_MOC_ViewerView
 #if !defined(Q_MOC_OUTPUT_REVISION)
 #define Q_MOC_OUTPUT_REVISION 9
 #elif Q_MOC_OUTPUT_REVISION != 9
