@@ -48,7 +48,7 @@ typedef struct _PangoContextClass PangoContextClass;
  * need to create a subclass of these, mail otaylor@redhat.com
  */
 
-GType         pango_context_get_type      (void);
+GType         pango_context_get_type      (void) G_GNUC_CONST;
 PangoContext *pango_context_new           (void);
 void          pango_context_add_font_map  (PangoContext                 *context,
 					   PangoFontMap                 *font_map);
@@ -77,10 +77,13 @@ PangoDirection        pango_context_get_base_dir         (PangoContext
  * consistent shaping/language engine and bidrectional level.
  * Returns a GList of PangoItem's
  */
-GList *pango_itemize (PangoContext   *context, 
-		      const char     *text, 
-		      gint            length,
-		      PangoAttrList  *attrs);
+GList *pango_itemize (PangoContext      *context,
+                      const char        *text,
+                      int                start_index,
+                      int                length,
+                      PangoAttrList     *attrs,
+                      PangoAttrIterator *cached_iter);
+
 
 #ifdef __cplusplus
 }
