@@ -31,6 +31,8 @@
 #include "pango-intset.h"
 #include "modules.h"
 
+#define PANGO_X_UNKNOWN_FLAG 0x10000000
+
 #define PANGO_LIGATURE_HACK_DEBUG
 
 #include <config.h>
@@ -442,7 +444,92 @@ pango_x_render  (Display           *display,
   
   for (i=0; i<glyphs->num_glyphs; i++)
     {
-      if (glyphs->glyphs[i].glyph)
+      if (glyphs->glyphs[i].glyph &
+          PANGO_X_UNKNOWN_FLAG)
+        {
+          int x1, y1, x2, y2; /* rectangle the character should go inside. */
+          int baseline;
+          PangoFontMetrics metrics;
+          gunichar wc;
+          
+          pango_font_get_metrics (font, NULL, &metrics);
+
+          x1 = x + (x_off + glyphs->glyphs[i].geometry.x_offset) / PANGO_SCALE;
+          y1 = y + (glyphs->glyphs[i].geometry.y_offset - metrics.ascent) / PANGO_SCALE;
+          x2 = x1 + glyphs->glyphs[i].geometry.width / PANGO_SCALE;
+          y2 = y1 + (metrics.ascent + metrics.descent) / PANGO_SCALE;
+          baseline = y1 + metrics.ascent / PANGO_SCALE;
+          
+          wc = glyphs->glyphs[i].glyph & (~PANGO_X_UNKNOWN_FLAG);
+
+          switch (wc)
+            {
+            case '\n':
+            case '\r':
+            case 0x2028: /* Line separator */
+            case 0x2029: /* Paragraph separator */
+              {
+                /* Draw a carriage-return thingy */
+                PangoRectangle up_stroke;
+                PangoRectangle across_stroke;
+
+		int stroke_thick = MAX ((y2 - y1) * 0.075, 1);
+		int hborder = (x2 - x1) * 0.1;
+		int arrow_height = 0.25 * (y2 - y1);
+		int top_border = (y2 - y1) * 0.25;
+
+		int arrow_x, arrow_width, tmp_height;
+
+		/* Draw the arrow-head */
+
+		tmp_height = (stroke_thick % 2 == 0) ? 2 : 1; /* Starting height */
+		arrow_height = 2 * ((1 + arrow_height - tmp_height) / 2) + tmp_height; /* Force symmetry */
+		arrow_width = 2 + arrow_height - tmp_height;
+
+		for (arrow_x = x1 + hborder; arrow_x < x1 + hborder + arrow_width; arrow_x++)
+		  {
+                    XDrawLine (display, d, gc,
+                               arrow_x,
+			       baseline - stroke_thick + (stroke_thick - tmp_height) / 2,
+                               arrow_x,
+			       baseline - stroke_thick + (stroke_thick - tmp_height) / 2 + tmp_height - 1);
+		    
+		    if ((arrow_x - x1 - hborder) % 2 == 1)
+		      tmp_height += 2;
+		  }
+
+                across_stroke.x = arrow_x;
+                across_stroke.width = x2 - hborder - arrow_x - stroke_thick;
+                across_stroke.y = baseline - stroke_thick;
+                across_stroke.height = stroke_thick;
+
+                XFillRectangle (display, d, gc,
+                                across_stroke.x, across_stroke.y,
+                                across_stroke.width, across_stroke.height);
+
+		up_stroke.x = across_stroke.x + across_stroke.width;
+		up_stroke.width = stroke_thick;
+		up_stroke.y = y1 + top_border;
+		up_stroke.height = baseline - up_stroke.y;
+
+                XFillRectangle (display, d, gc,
+                                up_stroke.x, up_stroke.y,
+                                up_stroke.width, up_stroke.height);
+              }
+              break;
+
+            default:
+
+              /* Here we should draw the box-with-numbers as in the
+               * Xft backend. The shaper never gives us a glyph that
+               * activates this case at the moment though, so also
+               * needs hacking.
+               */
+              
+              break;
+            }
+        }
+      else if (glyphs->glyphs[i].glyph)
 	{
 	  guint16 index = PANGO_X_GLYPH_INDEX (glyphs->glyphs[i].glyph);
 	  guint16 subfont_index = PANGO_X_GLYPH_SUBFONT (glyphs->glyphs[i].glyph);
@@ -486,7 +573,52 @@ pango_x_font_get_glyph_extents  (PangoFont      *font,
   XCharStruct *cs;
   PangoXSubfontInfo *subfont;
 
-  if (glyph && pango_x_find_glyph (font, glyph, &subfont, &cs))
+  if (glyph & PANGO_X_UNKNOWN_FLAG)
+    {
+      gunichar wc;
+      
+      wc = glyph & (~PANGO_X_UNKNOWN_FLAG);
+          
+      switch (wc)
+        {
+        case '\n':
+        case '\r':
+        case 0x2028: /* Line separator */
+        case 0x2029: /* Paragraph separator */
+          {
+            /* Size of carriage-return thingy */
+            PangoFontMetrics metrics;
+            int w;
+
+            pango_font_get_metrics (font, NULL, &metrics);
+            
+#define MAGIC_FACTOR 1.75
+            
+            w = metrics.approximate_char_width * MAGIC_FACTOR;
+            
+            if (ink_rect)
+              {
+                ink_rect->x = 0;
+                ink_rect->width = w;
+                ink_rect->y = - metrics.ascent;
+                ink_rect->height = metrics.ascent + metrics.descent;
+              }
+            if (logical_rect)
+              {
+                logical_rect->x = 0;
+                logical_rect->width = w;
+                logical_rect->y = - metrics.ascent;
+                logical_rect->height = metrics.ascent + metrics.descent;
+              }
+          }
+          break;
+          
+        default:
+              
+          break;
+        }
+    }
+  else if (glyph && pango_x_find_glyph (font, glyph, &subfont, &cs))
     {
       if (ink_rect)
 	{
@@ -557,7 +689,7 @@ get_font_metrics_from_subfonts (PangoFont        *font,
   GSList *tmp_list = subfonts;
   gboolean first = TRUE;
   int total_avg_widths = 0;
-  int n_avg_widths = 0;
+  int n_avg_widths = 0;  
   Atom avg_width_atom;
 
   avg_width_atom = pango_x_fontmap_atom_from_name (xfont->fontmap,
@@ -590,9 +722,24 @@ get_font_metrics_from_subfonts (PangoFont        *font,
 		}
 	    }
 
-          if (!get_int_prop (avg_width_atom, fs, &avg_width))
-            avg_width = (fs->min_bounds.width + fs->max_bounds.width) / 2;
+          if (get_int_prop (avg_width_atom, fs, &avg_width))
+            {
+              /* convert decipoints --> pango units.
+               * Resolution is in (points * PANGO_SCALE) / pixel,
+               * avg_width in decipoints.
+               * We want pixels * PANGO_SCALE
+               */
 
+              /* Convert to points * PANGO_SCALE */
+              avg_width *= PANGO_SCALE / (double) 10.0;
+              /* Convert to pixels * PANGO_SCALE */
+              avg_width *= (PANGO_SCALE / PANGO_X_FONT_MAP (PANGO_X_FONT (font)->fontmap)->resolution);
+            }
+          else
+            {
+              avg_width = PANGO_SCALE * ((fs->min_bounds.width + fs->max_bounds.width) / 2);
+            }
+          
           total_avg_widths += avg_width;
           n_avg_widths += 1;
 	}
@@ -603,7 +750,10 @@ get_font_metrics_from_subfonts (PangoFont        *font,
     }
 
   /* This is pretty darn bogus. */
-  metrics->approximate_char_width = total_avg_widths / n_avg_widths;
+  if (n_avg_widths)
+    metrics->approximate_char_width = total_avg_widths / n_avg_widths;
+  else
+    metrics->approximate_char_width = 10 * PANGO_SCALE;
 }
 
 /* Get composite font metrics for all subfonts resulting from shaping
@@ -628,6 +778,7 @@ get_font_metrics_from_string (PangoFont        *font,
   guint8 *embedding_levels;
   PangoDirection base_dir = PANGO_DIRECTION_LTR;
   GSList *subfonts = NULL;
+  gboolean finished = FALSE;
   
   text_ucs4 = g_utf8_to_ucs4_fast (str, -1, &n_chars);
   if (!text_ucs4)
@@ -640,22 +791,32 @@ get_font_metrics_from_string (PangoFont        *font,
 
   last_shaper = NULL;
   last_level = 0;
-  
+
   i = 0;
   p = start = str;
-  while (*p)
+  while (*p || !finished)
     {
-      gunichar wc = g_utf8_get_char (p);
-      p = g_utf8_next_char (p);
+      gunichar wc;
+
+      if (*p)
+	{
+	  wc = g_utf8_get_char (p);
+	  shaper = pango_font_find_shaper (font, lang, wc);
+	}
+      else
+	{
+	  finished = TRUE;
+	  shaper = NULL;
+	}
 	  
-      shaper = pango_font_find_shaper (font, lang, wc);
-      if (p > start &&
-	  (shaper != last_shaper || last_level != embedding_levels[i]))
+      if (p > start && 
+	  (finished ||
+	   (shaper != last_shaper || last_level != embedding_levels[i])))
 	{
 	  PangoAnalysis analysis;
 	  int j;
 
-	  analysis.shape_engine = shaper;
+	  analysis.shape_engine = last_shaper;
 	  analysis.lang_engine = NULL;
 	  analysis.font = font;
 	  analysis.level = last_level;
@@ -672,6 +833,9 @@ get_font_metrics_from_string (PangoFont        *font,
 	  start = p;
 	}
 
+      if (!finished)
+	p = g_utf8_next_char (p);
+
       last_shaper = shaper;
       last_level = embedding_levels[i];
       i++;
@@ -789,14 +953,14 @@ pango_x_font_get_metrics (PangoFont        *font,
 
       xfont->metrics_by_lang = g_slist_prepend (xfont->metrics_by_lang, info);
       
-      get_font_metrics_from_string (font, lang, str, &info->metrics);
+      get_font_metrics_from_string (font, lookup_lang, str, &info->metrics);
 
       /* This is sort of a sledgehammer solution, but we cache this
        * stuff so not a huge deal hopefully. Get the avg. width of the
        * chars in "0123456789"
        */
       context = pango_x_get_context (pango_x_fontmap_get_display (xfont->fontmap));
-      pango_context_set_lang (context, lang);
+      pango_context_set_lang (context, lookup_lang);
       layout = pango_layout_new (context);
       pango_layout_set_text (layout, "0123456789", -1);
 
@@ -1952,3 +2116,23 @@ pango_x_fallback_shape (PangoFont        *font,
       p = g_utf8_next_char (p);
     }
 }
+
+PangoGlyph
+pango_x_font_get_unknown_glyph (PangoFont *font,
+                                gunichar   wc)
+{
+  g_return_val_if_fail (PANGO_IS_FONT (font), 0);
+
+  switch (wc)
+    {
+    case '\n':
+    case '\r':
+    case 0x2028: /* Line separator */
+    case 0x2029: /* Paragraph separator */
+      return PANGO_X_UNKNOWN_FLAG | wc;
+      break;
+    default:
+      return 0;
+      break;
+    }
+}
